//********************************************************************
//*** ResourcePool/ResourcePoolStreamLogger.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePoolStreamLogger.hh,v 1.6 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_STREAMLOGGER_HH
#define RESOURCEPOOL_STREAMLOGGER_HH

#include <iostream>
#include <time.h>

#include "POSIX/sys/timeval.hh"
#include "ResourcePool/ResourcePoolEventObserver.hh"

namespace fatalmind {

/**
 * A logging Event observer impelementation.
 *
 * This ResourcePoolEventLogger implementation writes all events to 
 * a ostream provided on construction (e.g. std::cout).
 *
 * Please refer to the \link select.cc select.cc \endlink example to see it's usage.
 */
class ResourcePoolStreamLogger: public ResourcePoolEventObserver {
	public:
		ResourcePoolStreamLogger(std::ostream& a_os) 
		: ResourcePoolEventObserver()
		, os(a_os)
		{
		}

		virtual void event(event_t eventID, const std::string& resourceID = NoResourceID, const std::string& message = "N/A") {
			if ((logevents & eventID) != 0) {
				if (logevents & Event.TIME_HIRES) {
					::POSIX::timeval t;
					struct timezone tz;
					gettimeofday(&t, &tz);
					os << t << ";" << getEventName(eventID) << ';' <<  resourceID << ';' << message << std::endl;
				} else {
					os << time(NULL) << ";" << getEventName(eventID) << ';' <<  resourceID << ';' << message << std::endl;
				}
			}
		};

		static ResourcePoolStreamLogger defaultLogger;

	protected:
		virtual Clone* DoClone() const {
			return new ResourcePoolStreamLogger(*this);
		}
	private:
		std::ostream& os;
};

} // namespace fatalmind
#endif
