//********************************************************************
//*** ResourcePool/SQL/Bind.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: Bind.hh,v 1.19 2009-03-09 08:12:34 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_BIND_HH
#define RESOURCEPOOL_SQL_BIND_HH

#include "ResourcePool/SQL/BindInterface.hh"

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#include <vector>

namespace fatalmind {
namespace SQL {

/**
 * This class defines the common interface for all SQL* classes
 * which do accept input data via placeholders.
 */
template<class VBC, template <class> class CT>
class Bind: public BindInterface {
	protected:
		typedef VBC _datat;
		typedef gc_ptr<VBC> _ptrt;
		typedef std::vector<_ptrt> _argst;
		_argst _args;
	public:
		virtual ~Bind() {
		}

		template<typename T>
		inline
		void bind(const unsigned int pos, const T& val, bool nul = false) {
		        gc_ptr<VBC> ptr(new CT<T>(val, nul)); // gc_ptr must be available during complete bindPtr call (passed by ref)
		        bindPtr(pos, ptr);
		}
	
		virtual void clearParameters();

		virtual void bindoutputoperator(std::ostream& str) const;
		static void bindoutputoperator(std::ostream& str, const _argst&);

	private:
		// the bindPtr function performs the real work
		// the bind() function above is inlined and just a wrapper
		// to this function, which is NOT a function tempalte
		// gc_ptr by ref, private and no threading at this point
		void bindPtr(const unsigned int pos, gc_ptr<VBC>& ptr);

};

} // namespace SQL
} // namespace fatalmind

#endif
