//********************************************************************
//*** ResourcePool/SQL/BindOut.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: BindOut.hh,v 1.15 2009-03-07 23:04:02 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_BINDOUT_HH
#define RESOURCEPOOL_SQL_BINDOUT_HH

#include "ResourcePool/SQL/BindOutInterface.hh"

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#include <vector>

namespace fatalmind {
namespace SQL {


/**
 * This class defines the common interface for all SQL* classes
 * which do return output colums.
 */
template<class VBC, template <class> class CT>
class BindOut: public BindOutInterface {
	public:
		virtual ~BindOut();

		template<typename T>
		inline
		void bindout(const unsigned int pos, T& val, bool& isnull) {
			gc_ptr<VBC> ptr(new CT<T>(val, isnull));
			bindPtr(pos, ptr);
		}

		template<typename T>
		inline
		void bindout(const unsigned int pos, T& val, BindOutNotNullConstraint = NOTNULL) {
			bindout(pos, val, BindOutInterface::notnullconstant);
		}

	protected:
		VBC& getOutParam(const unsigned int pos);

		typedef gc_ptr<VBC> _refT;
		typedef std::vector<_refT> _outparamt;
		_outparamt _outparam;

		typename _outparamt::const_iterator _begin;
		typename _outparamt::const_iterator _end;

		void setIterators() {
			_begin = _outparam.begin();
			_end   = _outparam.end();
		}
	private:
		void bindPtr(const unsigned int pos, gc_ptr<VBC> ptr);

		void resizeArgs(const unsigned int pos);
};


} // namespace SQL
} // namespace fatalmind
#endif
