//********************************************************************
//*** ResourcePool/SQL/NULL.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: NULL.t.cc,v 1.17 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

void test_NULL_init(RP& p)
{
	drop(p);
	init(p);
}

void test_NULL_insert(RP& p)
{
	RP::Commands::SQLExecute ins(p, "insert into hirsch (id, ukn) values (1000, null)");
	p.execute(ins);
	RP::Commands::SQLExecute ins2(p, "insert into hirsch (id, ukn) values (?, ?)");
	std::string nul("notnull");
	ins2.bind(0, 1001);
	ins2.bind(1, nul, true);
	p.execute(ins2);
	ins2.bind(0, 1002);
	ins2.bind(1, nul);
	p.execute(ins2);
}

void test_NULL_select(RP& p)
{
	string sql("select id, ukn from hirsch where id = ?");
	RP::Commands::SQLSelectRow cmd(p, sql);
	int col1;
	std::string col2("initialized");
	bool col1null, col2null;

	cmd.bindout(0, col1, col1null);
	cmd.bindout(1, col2, col2null);
	cmd.bind(0, 1000);
	p.execute(cmd);
	assertFalse(col1null);
	assertTrue(col2null);
	assertEqualT(int, col1, 1000);
	assertEqual(col2, "initialized"); // fetching null values may not change the value itself

	cmd.bind(0, 1001);
	p.execute(cmd);
	assertFalse(col1null);
	assertTrue(col2null);
	assertEqualT(int, col1, 1001);
	assertEqual(col2, "initialized"); // fetching null values may not change the value itself

	cmd.bind(0, 1002);
	p.execute(cmd);
	assertEqualT(int, col1, 1002);
	assertFalse(col1null);
	assertFalse(col2null);
	assertEqual(col2, "notnull"); // we actually fetched something
}

void test_NULL_select_CONSTRAINT(RP& p)
{
	string sql("select id, ukn from hirsch where id = ?");
	RP::Commands::SQLSelectRow cmd(p, sql);
	int col1;
	std::string col2("initialized");
	bool col2null;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2, col2null);
	cmd.bind(0, 1000);
	p.execute(cmd);
	assertTrue(col2null);
	assertEqualT(int, col1, 1000);
	assertEqual(col2, "initialized");

	cmd.bindout(1, col2);
	try {
		p.execute(cmd); // should throw
		assertNotReach();
	} catch (...) {
	}
	assertEqual(col2, "initialized");
}

void test_NULL_lob(RP& p) {
	try {
		RP::Commands::SQLExecute create_lob(p, getLobTable(p));
		p.execute(create_lob);
	} catch (...) {
		// it's hopefully "already exists"
		RP::Commands::SQLExecute del(p, "delete from fatalmind_lob");
		p.execute(del);
	}
	
	RP::Commands::SQLExecute insert(p, "insert into fatalmind_lob (id, val) values (?, ?)");
	insert.bind(0, 1);
	std::string col2("blah");
	insert.bind(1, col2, true);
	p.execute(insert);
	
	RP::Commands::SQLSelectRow select1(p, "select val from fatalmind_lob where id=?");
	std::string out("init");
	bool outnull;

	select1.bind(0, 1);
	select1.bindout(0, out, outnull);
	p.execute(select1);
	assertTrue(outnull);
	assertEqual(out, "init"); // unchanged

	// constraint
	select1.bindout(0, out);
	try {
		p.execute(select1);
	} catch (...) {
	}
	assertEqual(out, "init"); // unchanged

	try {
		RP::Commands::SQLExecute drop(p, "drop table fatalmind_lob");
		p.execute(drop);
	} catch (...) {
	}
}

void test_NULL_numeric(RP& p) {
	try {
 		RP::Commands::SQLExecute create_num(p, "create table fatalmind_numeric (id numeric not null primary key, val numeric)");
		p.execute(create_num);
	} catch (...) {
		// it's hopefully "already exists"
		RP::Commands::SQLExecute del(p, "delete from fatalmind_numeric");
		p.execute(del);
	}
	
	RP::Commands::SQLExecute insert(p, "insert into fatalmind_numeric (id, val) values (?, ?)");
	insert.bind(0, 1);
	insert.bind(1, 42, true);
	p.execute(insert);
	
	RP::Commands::SQLSelectRow select1(p, "select val from fatalmind_numeric where id=?");
	int numeric = 41;
	bool outnull;
	select1.bind(0, 1);
	select1.bindout(0, numeric, outnull);
	p.execute(select1);
	assertTrue(outnull);
	assertEqual(numeric, 41);

	// constraint testing
	select1.bindout(0, numeric);
	try {
		p.execute(select1);
		assertNotReach();
	} catch (...) {
	}
	assertEqual(numeric, 41);

	try {
		RP::Commands::SQLExecute drop(p, "drop table fatalmind_numeric");
		p.execute(drop);
	} catch (...) {
	}
}


void test_NULL_cleanup(RP& p) {
	drop(p);
}
