//********************************************************************
//*** ResourcePool/SQL/RTSQLSelect.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLSelect.hh,v 1.24 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RTSQLSELECT_HH
#define RESOURCEPOOL_SQL_RTSQLSELECT_HH

#include "mm/gc_ptr.hh"
#include "ResourcePool/SQL/RTSQLCommand.hh"
#include "ResourcePool/SQL/Resource.hh"
#include "ResourcePool/SQL/BindInterface.hh"
#include "ResourcePool/SQL/BindOutInterface.hh"
#include "ResourcePool/SQL/RTCommandParameters.hh"
#include "ResourcePool/SQL/RTSQLSelectRow.hh"
#include "ResourcePool/SQL/SQLFetcher.hh"

namespace fatalmind {
namespace SQL {

template<class TM> class RTSQLSelectRow_Impl;

/**
 * A command to issue a select statement against
 * an SQL database.
 *
 * The general workflow to issue an SQL select statement is:
 * - Implement the code which is to be executed for each row in a class derived from SQLFetcher
 * - Construct a new RTSQLSelect instance and provide an instance of your SQLFetcher class to it.
 * - Issue all required bind() and bindout() calls to setup the bind parameters
 * - Execute the command
 *
 * Please refer to the <a href="select_8cc-example.html">select.cc example</a> to see it in action.
 */
template<class TM>
class RTSQLSelect
: public RTSQLCommand<TM>
, public BindInterface
, public BindOutInterface
{
	public:
		RTSQLSelect(const ResourcePool<ResourceType<TM> >& p, const std::string& sql, SQL::SQLFetcher& fetcher);
		RTSQLSelect(const RTSQLSelect<TM>& rho);
		virtual ~RTSQLSelect();
		
		template<typename T>
		void bind(const unsigned int pos, const T& val, bool nul = false);

		template<typename T>
		void bindout(const unsigned int pos, T& val, bool& isnull);

		template<typename T>
		void bindout(const unsigned int pos, T& val, BindOutNotNullConstraint = NOTNULL);

		virtual void clearParameters();

//		bool isnull(const unsigned int pos) const;

		void execute(typename ResourceType<TM>::resource_t& hand);
		virtual void outputoperator(std::ostream&) const;

		virtual const Clone* getPlainSpecializedCommand() const;

	protected:
		virtual Clone* DoClone() const {
			return new RTSQLSelect<TM>(*this);
		}
	private:
		gc_ptr<RTSQLSelectRow_Impl<TM> > _realSQLExecute;	

		typedef typename SQLFactory<TM>::databaseType_t databaseType_t;
};


} // namespace SQL
} // namespace fatalmind
#endif
