//********************************************************************
//*** ResourcePool/oracle/CommandCommon.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: CommandCommon.cc,v 1.52 2009-03-09 08:12:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "CommandCommon.hh"
#include "Resource.hh"
#include "OracleExceptionFactory.hh"
#include "ResourcePool.hh"
#include "oracleType.cc"
#include "ResourcePool/Command.hh"
#include <sstream>

namespace fatalmind {
namespace oracle {

template<class TM, template<class> class SLMDC>
const std::string CommandCommon<TM, SLMDC>::_CommandCommonName("oracle::CommandCommon");

template<class TM, template<class> class SLMDC>
CommandCommon<TM, SLMDC>::CommandCommon(const std::string& SQL) 
: _SQL(SQL)
{
}

template<class TM, template<class> class SLMDC>
CommandCommon<TM, SLMDC>::~CommandCommon() {
}

template<class TM, template<class> class SLMDC>
void CommandCommon<TM, SLMDC>::outputoperator(std::ostream& s) const {
	s << getCommandName() << "[" << getSQL() << "|";
	bindoutputoperator(s);
 	s << "]";
}
template<class TM, template<class> class SLMDC>
const std::string& CommandCommon<TM, SLMDC>::getCommandName() const {
	return _CommandCommonName;
}

template<class TM, template<class> class SLMDC>
StatementCache::handle_t CommandCommon<TM, SLMDC>::prepare(oracleResource<TM>& hand, StatementCache& sthcache, const oracleStatement& oraSQL) {
	gc_ptr<EventReader> er = hand.getReader(hand.Event.PREPARE);
	StatementCache::handle_t sth = sthcache.open(oraSQL, true);
	if (sth->prepare(hand)) {
		(*er) << "CACHED;" << (*this);
	} else {
		(*er) << ';' << (*this);
	}
	return sth;
}

template<class TM, template<class> class SLMDC>
void CommandCommon<TM, SLMDC>::real_bind(oracleHandles& hand, OCIStatementHandle& sth) {
	sword ret;
	_argst::const_iterator end = _args.end();
	int pos = 0;
	for (_argst::const_iterator i = _args.begin(); i != end; ++i) {
		++pos;
		OCIBind  *bnd1p = NULL;
		_ptrt ii = *i;
		if (ii.get() != 0) {
			if ((ret = OCIBindByPos(sth.get()
				, &bnd1p
				, hand.getError()
				, pos 
				, ii->ptr()
				, ii->size()
				, ii->type()
				, ii->indptr()
				, (ub2 *)0
				, (ub2 *)0
				, 0
				, (ub4 *)0
				, OCI_DEFAULT)
			) != OCI_SUCCESS) {
				OracleExceptionFactory::throwit(hand.LogConstant.OCIBindByPos, hand, ret, _SQL);
			}
		} else {
			OracleExceptionFactory::throwit(hand.LogConstant.OCIBindByPos, "ORA", 1008, "not all params bound", _SQL);
		} 
	}
}

template<class TM, template<class> class SLMDC>
const std::string& CommandCommon<TM, SLMDC>::getSQL() const {
	return _SQL;
}

template class CommandCommon<DefaultThreadedModel, oracle::SelectListMetaDataCache >;

} // namespace oracle

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<char>
	(const unsigned int pos, const char& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<signed char>
	(const unsigned int pos, const signed char& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned char>
	(const unsigned int pos, const unsigned char& val, bool nul);

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<short int>
	(const unsigned int pos, const short int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<int>
	(const unsigned int pos, const int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<long int>
	(const unsigned int pos, const long int& val, bool nul);

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned short int>
	(const unsigned int pos, unsigned const short int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned int>
	(const unsigned int pos, unsigned const int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned long int>
	(const unsigned int pos, unsigned const long int& val, bool nul);

// wchar_t, bool, not supported..

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<float>
	(const unsigned int pos, const float& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<double>
	(const unsigned int pos, const double& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<long double>
	(const unsigned int pos, const long double& val, bool nul);

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<std::string>
	(const unsigned int pos, const std::string& val, bool nul);


} // namespace fatalmind

