//********************************************************************
//*** ResourcePool/oracle/CommandCommon.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: CommandCommon.hh,v 1.37 2009-03-08 16:45:53 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_COMMANDCOMMON_HH
#define RESOURCEPOOL_ORACLE_COMMANDCOMMON_HH

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_HH
#include "CommandInterface.hh"
#endif

#include "oracleType.hh"

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#ifndef RESOURCEPOOL_COMMAND_HH
#include "ResourcePool/Command.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIHANDLEFWD_HH
#include "OCIHandleFwd.hh"
#endif

#ifndef RESOURCEPOOL_SQL_BIND_HH
#include "ResourcePool/SQL/Bind.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_STATEMENTCACHE_HH
#include "StatementCache.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLESTATEMENT_HH
#include "oracleStatement.hh"
#endif

#include <string>

namespace fatalmind {
namespace oracle {
/**
 * Defines the methods common to all oracle commands.
 */
template<class TM, template<class> class SLMDC>
class CommandCommon
: public CommandInterface<TM, SLMDC>
, public SQL::Bind<oracleType, oracleTypeImpl> 
{
	public:
		virtual ~CommandCommon();
		
		virtual void outputoperator(std::ostream& s) const;
	protected:
		typedef SQL::Bind<oracleType, oracleTypeImpl> super_bind;
		/**
		 * CommandCommon can not exits allone, therefor the ctor is
		 * protected.
		 */
		CommandCommon(const std::string&);

		/*
		 * this functions is the primary target to be overwritten by
		 * the respective SQL actions (SQLExecute, SQLSelect and so on)
		 */
		virtual void _execute(oracleResource<TM>&, StatementCache&) = 0;

		/*
		 * performs the OCI prepare call (cached)
		 */
		StatementCache::handle_t prepare(oracleResource<TM>&, StatementCache&, const oracleStatement& oraSQL);
		/*
		 * perform the OCIBind calls. since this is required by all (not only)
		 * select calls its here in the Common code
		 */
		void real_bind(oracleHandles&, OCIStatementHandle& sth);

		const std::string& getSQL() const;

		virtual const std::string& getCommandName() const;
		static const std::string _CommandCommonName;
	
		/**
		 * this is a copy of the original SQL statement.
		 * since oracle doesn't support '?' placeholders we will do the
		 * parsing our selves. so this is actually not the SQL sent to oracle
		 */
		std::string _SQL;
};

} // namespace oracle
} // namespace fatalmind
#endif
