//********************************************************************
//*** ResourcePool/oracle/SQLSelectRow.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelectRow.cc,v 1.63 2009-03-08 16:45:53 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_ORACLE_SQLSELECTROW_HH
#include "SQLSelectRow.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEREFTYPE_HH
#include "oracleRefType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH
#include "SelectListMetaData.hh"
#endif

#include "ResourcePool/SQL/BindOut.hh"

#include "oracleRefType.cc"

#include "SelectListMetaDataCache.hh"

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_CC
#include "CommandInterface.cc"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCEPOOL_HH
#include "ResourcePool.hh"
#endif

#include <sstream>

namespace fatalmind {
namespace oracle {

template<class TM, template<class> class SLMDC>
const std::string SQLSelectRow<TM, SLMDC>::_SQLSelectRowName("oracle::SQLSelectRow");


template<class TM, template<class> class SLMDC>
SQLSelectRow<TM, SLMDC>::SQLSelectRow(const std::string& SQL, SQL::SQLFetcher& fetcher) 
: CommandCommon<TM, SLMDC>(SQL)
, SQLSelect(fetcher)
{
}

template<class TM, template<class> class SLMDC>
SQLSelectRow<TM, SLMDC>::SQLSelectRow(const ResourcePool<ResourceType<TM> >& p, const std::string& SQL, SQL::SQLFetcher& fetcher) 
: CommandCommon<TM, SLMDC>(SQL)
, SQLSelect(fetcher)
{
}

template<class TM, template<class> class SLMDC>
Clone* SQLSelectRow<TM, SLMDC>::DoClone() const {
	return new SQLSelectRow<TM, SLMDC>(*this);
}

template<class TM, template<class> class SLMDC>
SQLSelectRow<TM, SLMDC>::~SQLSelectRow()
{
}

sb4 cdf_fetch_buffer(dvoid *vdmd
	, OCIDefine *defnp
	, ub4 iter
	, dvoid **bufpp
	, ub4 **alenpp
	, ub1 *piecep
	, dvoid **indpp 
	, ub2 **rcpp
) {
	DefineMetaData* dmd = (DefineMetaData*)vdmd;
	if(piecep[0]!=(ub1)OCI_FIRST_PIECE) {
		dmd->done(**alenpp, *bufpp);
	}
	*bufpp = dmd->getDataPtr();
	dmd->resetSize();
	*alenpp = dmd->getSizePtr_ub4();
	*indpp  = dmd->getIndPtr();

	*rcpp = 0;
	return OCI_CONTINUE;
}

void oracleBindOut::real_bindout(oracleHandles& hand, const OCIStatementHandle& sth, const std::string& SQL) {
	sword ret;
	int pos = 0;
	for (_outparamt::const_iterator i = _begin; i != _end; ++i) {
		++pos;
		OCIDefine *defhp = NULL;
		DefineMetaData* dmd = _slb->getDefineMetaData(pos-1);
		const ColumnMetaData& cmd = _slmd->getColumnMetaData(pos-1);
		// Def.: Buffered fetch: to fetch into the SelectListBuffer and copy afterwards over to the bindout()ed veriable.
		// (vs. non-bufferd, directly tell the address of the users variable to OCI)
		// 
		// a non-buffered fetch is only possible for numeric types
		//bool buffered = ((cmd.getType() != SQLT_NUM) ||((*i)->type() != SQLT_NUM));
		bool buffered = ColumnMetaData::needsBuffer((*i)->type());
		bool piecewise = cmd.piecewise();	
		ub2*  sizeptr = 0;
		void* dataptr = 0;
		sb4*  indptr  = 0;
		sb4   size    = 0;
		ub4   mode    = OCI_DEFAULT;
		if (buffered) {
			if (!piecewise) {
				dataptr = dmd->getDataPtr();
				size    = cmd.getSize();
				indptr  = (sb4*)(*i)->indptr();
				sizeptr = dmd->getSizePtr_ub2();
			} else {
				size    = std::numeric_limits<sb4>::max();
				dmd->setIndPtr((sb2*)(*i)->indptr());
				mode   |= OCI_DYNAMIC_FETCH;
			}
		} else {
			dataptr = (*i)->ptr();
			size    = (*i)->capacity();
			indptr  = (sb4*)(*i)->indptr();
		}
		if ((ret = OCIDefineByPos(sth.get()
			, &defhp
			, hand.getError()
			, pos
			, dataptr
			, size
			, (*i)->type()
			, indptr
			, sizeptr
			, (ub2 *) 0
			, mode
		)) != OCI_SUCCESS) {
			std::ostringstream os;
			os << "OCIDefineByPos(" << pos << ";" << (*i)->capacity() << ")";
			OracleExceptionFactory::throwit(os.str(), hand, ret, SQL);
		}
		if ((mode & OCI_DYNAMIC_FETCH) == OCI_DYNAMIC_FETCH) {
			if ((ret = OCIDefineDynamic(defhp
				, hand.getError()
				, dmd
				, cdf_fetch_buffer
			)) != OCI_SUCCESS) {
				OracleExceptionFactory::throwit("OCIDefineDynamic", hand, ret, SQL);
			}
		}
	}
}

void oracleBindOut::validateBindOut(const std::string& SQL) const {
	for (_outparamt::const_iterator i = _begin; i != _end; ++i) {
		if ((*i).get() == 0) {
			std::ostringstream os;
			os << "(OCIDefineByPos): bindout not called for column " << std::distance(_begin, i);
			throw NFOOracleException(os.str(), "ORA-01008", 1008, SQL); // TODO, constants
		}
	}
}

oracleBindOut::~oracleBindOut() {
}

/*
 * allocates a SelectListBuffer if the statement is found
 * in SelectListMetaData Cache.
 * @returns true if a buffer was allocated, false otherwise.
 */
template<class TM, template<class> class SLMDC>
void SQLSelectRow<TM, SLMDC>::preSelectBuffer(oracleResource<TM>& hand)
{
	if (_slb.isNull()) {
		_slmd = hand.getSelectListMetaDataCache().get(_SQL);
		if (! _slmd.isNull()) {
			_slb.reset(new SelectListBuffer(_slmd, *this));
		}
	}
}

template<class TM, template<class> class SLMDC>
void SQLSelectRow<TM, SLMDC>::preSelectBufferBind(oracleHandles& hand, OCIStatementHandle& sth)
{
	if (! _slb.isNull()) {
		real_bindout(hand, sth, _SQL);
	}
}

template<class TM, template<class> class SLMDC>
void SQLSelectRow<TM, SLMDC>::postSelectBuffer(oracleResource<TM>& hand, OCIStatementHandle& sth)
{
	if (_slb.isNull()) {
		_slmd = hand.getSelectListMetaDataCache().get(_SQL, SelectListMetaDataCtor(_SQL, sth, hand));
		_slb.reset(new SelectListBuffer(_slmd, *this));
		real_bindout(hand, sth, _SQL);
	}
}

/*
 * prepares the fetchbuffer.
 * @returns the open statement handle
 */
template<class TM, template<class> class SLMDC>
StatementCache::handle_t SQLSelectRow<TM, SLMDC>::prepare_buffer(oracleResource<TM>& hand, StatementCache& sthcache)
{
	preSelectBuffer(hand);
	StatementCache::handle_t sth;
	if (_slb.isNull()) {
		sth = prepare(hand, sthcache, oracleStatement(_SQL));
	} else {
		sth = prepare(hand, sthcache, _slmd->getOraSQL());
	}
	return sth;
}

/*
 * Prepares everything for an selct to be done.
 * -> isssues prepare_buffer and will also return it's result
 * -> issues the real_bind to call the OCI bind functions
 * -> does the preSelectBuffer
 */
template<class TM, template<class> class SLMDC>
StatementCache::handle_t SQLSelectRow<TM, SLMDC>::prepareForSelect(oracleResource<TM>& hand, StatementCache& sthcache)
{
	StatementCache::handle_t sth = prepare_buffer(hand, sthcache);
	real_bind(hand, *sth);
	preSelectBufferBind(hand, *sth);

	return sth;
}

template<class TM, template<class> class SLMDC>
void SQLSelectRow<TM, SLMDC>::_execute(oracleResource<TM>& hand, StatementCache& sthcache) 
{
	sword ret;

	validateBindOut(_SQL);

	StatementCache::handle_t sth = prepareForSelect(hand, sthcache);

	// buffer already prepared?, we can perform an exact fetch
	register bool exact = ! _slb.isNull();
	if (exact) { 
		_start();
	}

	/* execute statement */
	if ((ret = OCIStmtExecute(hand.getSvcCtx()
		, sth->get()
		, hand.getError()
		, (ub4) (exact? 1 : 0)
		, (ub4) 0
		, (CONST OCISnapshot *) 0
		, (OCISnapshot *) 0
		, (ub4) (exact ? OCI_EXACT_FETCH : OCI_DEFAULT) | ((hand.getTxDepth() == 0) ? OCI_COMMIT_ON_SUCCESS : OCI_DEFAULT))
	) != OCI_SUCCESS) {
		OracleExceptionFactory oef(hand.LogConstant.OCIStmtExecute, hand.getError(), ret, _SQL);
		gc_ptr<EventReader> er = hand.getReader(hand.Event.EXECUTE);
		(*er) << oef.getErrorCode() << hand.LogConstant.SEMICOLON << (*this);

		if (oef.getErrorNumber() == 1406) {
			// invalidate cache, so that re-execution will not cause the same problem
			hand.getSelectListMetaDataCache().invalidate(_SQL);
			_slmd.reset(0);
			_slb.reset(0);
		} 
		
		oef.throwit();
	}
	{
		gc_ptr<EventReader> er = hand.getReader(hand.Event.EXECUTE);
		(*er) <<  (exact ? hand.LogConstant.EXACT : hand.LogConstant.EMPTY ) << hand.LogConstant.SEMICOLON << (*this);
	}

	postSelectBuffer(hand, *sth);

	if (!exact) {
		internal::registerFinish fin(hand, *this, *sth, _SQL);
		gc_ptr<EventReader> er = hand.getReader(hand.Event.FETCH);
		if (!fetch(hand, *sth, _SQL)) {
			(*er) << hand.LogConstant.NODATA << (*this);
			throw NFOOracleException(hand.LogConstant.OCI_NODATA, hand.LogConstant.ORA99999, 999999, _SQL);
		}

		if (_fetch(hand, *sth, _SQL)) {
			(*er) << hand.LogConstant.MOREDATA << (*this);
			throw NFOOracleException(hand.LogConstant.ORA01422MSG, hand.LogConstant.ORA01422, 1422, _SQL);
		}
	} else {
		_done();
	}
}

template<class TM, template<class> class SLMDC>
const std::string& SQLSelectRow<TM, SLMDC>::getCommandName() const {
	return _SQLSelectRowName;
}

void oracleBindOut::_start() {
	for (_outparamt::const_iterator i = _begin
		; i != _end
		; ++i
	) {
		const _refT& x = *i;
		if (x.get() != 0) {
			x->start();
		}
	}
}

void oracleBindOut::_done() {
	int pos = 0;
	SelectListBuffer& b_slb = *_slb; // de-refernece only once
	for (_outparamt::const_iterator i = _begin
		; i != _end
		; ++i, ++pos
	) {
		DefineMetaData* dmd = b_slb.getDefineMetaData(pos);
		if ((*i).get() != 0 && (dmd != 0)) {
			if (b_slb.piecewiseFetch(pos)) { // only the sizetype differs
				(*i)->done(dmd->getSize_ub4(), dmd->getDataPtr());
			} else {
				(*i)->done(dmd->getSize_ub2(), dmd->getDataPtr());
			}
		}
	}
}


/*
 * \ret false if no data were fetched
 * 
 */
bool oracleBindOut::_fetch(oracleHandles& hand
	, const OCIStatementHandle& sth
	, const std::string& SQL
) {
	sword ret;
	if ((ret = OCIStmtFetch2( sth.get()
		, hand.getError()
		, 1
		, OCI_DEFAULT
		, 0
		, OCI_DEFAULT)
	) != OCI_SUCCESS) {
		if (ret == OCI_NO_DATA) {
			return false;
		} else {
			OracleExceptionFactory::throwit("OCIStmtFetch2", hand, ret, SQL);
		}
	}
	return true;
}

bool oracleBindOut::fetch(oracleHandles& hand
	, const OCIStatementHandle& sth
	, const std::string& SQL
) {
	_start();
	bool fetched = _fetch(hand, sth, SQL);
	if (fetched) {
		// call done only if something was fetched,
		// e.g. don't call it in case an error happend
		_done();
	}
	return fetched;
}

void oracleBindOut::finish(oracleHandles& hand
	, const OCIStatementHandle& sth
	, const std::string& SQL
) {
	sword ret;
	if ((ret = OCIStmtFetch2( sth.get()
		, hand.getError()
		, 0
		, OCI_DEFAULT
		, 0
		, OCI_DEFAULT)
	) != OCI_SUCCESS) {
		if (ret != OCI_NO_DATA) {
			OracleExceptionFactory::throwit("OCIStmtFetch2(finish)", hand, ret, SQL);
		}
	}
}

template class
SQLSelectRow<DefaultThreadedModel, oracle::SelectListMetaDataCache>;

//template class
//SQLSelectRow<DefaultThreadedModel, oracle::SelectListMetaDataNopCache>;

} // namespace oracle

template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<char>
    (const unsigned int pos, char& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<char>
    (const unsigned int pos, char& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<signed char>
    (const unsigned int pos, signed char& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<signed char>
    (const unsigned int pos, signed char& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned char>
    (const unsigned int pos, unsigned char& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned char>
    (const unsigned int pos, unsigned char& val, BindOutNotNullConstraint);

template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<short int>
    (const unsigned int pos, short int& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<short int>
    (const unsigned int pos, short int& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<int>
    (const unsigned int pos, int& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<int>
    (const unsigned int pos, int& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<long int>
    (const unsigned int pos, long int& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<long int>
    (const unsigned int pos, long int& val, BindOutNotNullConstraint);

template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned short int>
    (const unsigned int pos, unsigned short int& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned short int>
    (const unsigned int pos, unsigned short int& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned int>
    (const unsigned int pos, unsigned int& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned int>
    (const unsigned int pos, unsigned int& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned long int>
    (const unsigned int pos, unsigned long int& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<unsigned long int>
    (const unsigned int pos, unsigned long int& val, BindOutNotNullConstraint);

template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<float>
    (const unsigned int pos, float& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<float>
    (const unsigned int pos, float& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<double>
    (const unsigned int pos, double& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<double>
    (const unsigned int pos, double& val, BindOutNotNullConstraint);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<long double>
    (const unsigned int pos, long double& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<long double>
    (const unsigned int pos, long double& val, BindOutNotNullConstraint);

template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<std::string>
    (const unsigned int pos, std::string& val, bool& isnull);
template void SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>::bindout<std::string>
    (const unsigned int pos, std::string& val, BindOutNotNullConstraint);

} // namespace fatalmind
