//********************************************************************
//*** ResourcePool/oracle/SQLSelectRow.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelectRow.hh,v 1.42 2009-03-08 16:45:53 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_SQLSELECTROW_HH
#define RESOURCEPOOL_ORACLE_SQLSELECTROW_HH

#include "mm/gc_ptr.hh"

#include "CommandCommon.hh"
#include "ResourceFwd.hh"

#include "ResourcePool/SQL/BindOut.hh"
#include "ResourcePool/SQL/SQLFetcher.hh"
#include "ResourcePool/SQL/SQLSelect.hh"

#include "oracleRefTypeFwd.hh"

#include <string>

namespace fatalmind {
namespace oracle {

// TODO: forward decl...
class SelectListMetaData;
class SelectListBuffer;
namespace internal {

class registerFinish;
};

/**
 * blah
 */
class oracleBindOut
: public SQL::BindOut<oracleRefType, oracleRefTypeImpl>
{
	public:
		using SQL::BindOut<oracleRefType, oracleRefTypeImpl>::NOTNULL;
	protected:
		~oracleBindOut();

		void _done();
		void _start();
		void real_bindout(oracleHandles&, const OCIStatementHandle&, const std::string&);

		bool fetch(oracleHandles&, const OCIStatementHandle&, const std::string&);
		bool _fetch(oracleHandles&, const OCIStatementHandle&, const std::string&);
		void finish(oracleHandles&, const OCIStatementHandle&, const std::string&);

		gc_ptr<SelectListMetaData> _slmd;
		gc_ptr<SelectListBuffer> _slb;

		void validateBindOut(const std::string& SQL) const;

	friend 
		class SelectListBuffer;	
	friend
		class internal::registerFinish;
};

/**
 * Select a single row from an oracle database
 */
template<class TM, template<class> class SLMDC>
class SQLSelectRow
: public CommandCommon<TM, SLMDC>
, public oracleBindOut 
, public fatalmind::SQLSelect
{
		typedef CommandCommon<TM, SLMDC> super;
	protected:
		using super::_SQL;
		using oracleBindOut::_slb;
	public:
		using oracleBindOut::NOTNULL;

		SQLSelectRow(const std::string& SQL, SQL::SQLFetcher& = SQL::NopFetcher);
		SQLSelectRow(const ResourcePool<ResourceType<TM> >& p, const std::string& SQL, SQL::SQLFetcher& = SQL::NopFetcher);
		virtual ~SQLSelectRow();

		using super::execute;

	protected:
		virtual Clone* DoClone() const;
		void _execute(oracleResource<TM>&, StatementCache&);

		inline void postSelectBuffer(oracleResource<TM>&, OCIStatementHandle& sth);
		inline void preSelectBuffer(oracleResource<TM>&);
		inline void preSelectBufferBind(oracleHandles&, OCIStatementHandle& sth);
		inline StatementCache::handle_t prepare_buffer(oracleResource<TM>& hand, StatementCache& sthcache);
		StatementCache::handle_t prepareForSelect(oracleResource<TM>& hand, StatementCache& sthcache);
		

		virtual const std::string& getCommandName() const;
		static const std::string _SQLSelectRowName;
};

namespace internal {
// the best way to make a class really private ;)
class registerFinish {
	public:
		registerFinish(oracleHandles& hand, oracleBindOut& cmd, const OCIStatementHandle& sth, const std::string& SQL)
		: _hand(hand)
		, _sth(sth)
		, _SQL(SQL)
		, _cmd(cmd)
		{
		}
		~registerFinish() {
			_cmd.finish(_hand, _sth, _SQL);
		};
	private:
		oracleHandles& _hand;
		const OCIStatementHandle &_sth;
		const std::string& _SQL;
		
		oracleBindOut&  _cmd;
};

} // namespace internal
} // namespace oracle
} // namespace fatalmind
#endif
